#!/usr/bin/env Rscript

#Get arguments
suppressMessages(library(optparse))
option_list = list(
  make_option(c("-a", "--abundances"), type="character", help="abundance file generated by StrainR", metavar="character"),
  make_option(c("-c", "--weightedpercentile"), type="numeric", default=60, help="Weighted percentile for a strain's FUKMs to use in abundance estimation", metavar="numeric"),
  make_option(c("-s", "--subcontigfilter"), type="numeric", default=0, help="Percentage of a strain's subcontigs that should be filtered out based on number of unique k-mers", metavar="numeric"),
  make_option(c("-i", "--indir"), type="character", help="directory outputted by PreProcessR", metavar="character"),
  make_option(c("-p", "--prefix"), type="character", default="sample", help="the same prefix as used by StrainR", metavar="character")
)

opt_parser = OptionParser(option_list=option_list)
opt = parse_args(opt_parser)
if (is.null(opt$indir)){
  stop("Error in Plot.R: The output directory generated by PreProcessR needs to be provided after -i option. Use --help option for more info", call.=FALSE)
}
if (is.null(opt$abundances)){
  stop("Error in Plot.R: The output directory generated by StrainR needs to be provided after -a option. Use --help option for more info", call.=FALSE)
}

suppressMessages(library(tidyverse))

#Get mappings and merge with subcontig counts
message(date(), " Normalizing Data for ", opt$prefix)

mapped<-system(paste0("grep '#Mapped' ",opt$abundances, "/",opt$prefix,".rpkm"), intern = TRUE) %>% gsub("#Mapped\t","", .) %>% as.numeric()
Norm<-
  read_tsv(paste0(opt$abundances, "/",opt$prefix,".rpkm"), skip=4) %>% 
  dplyr::rename(SubcontigID=`#Name`) %>%
  select(-Length) %>%
  right_join(
    read_tsv(paste0(opt$indir,"/KmerContent.report"), col_types="ccccid")
  ) %>%
  mutate(Total_Mapped_Reads_In_Sample=mapped) %>%
  select(StrainID, ContigID, Start_Stop, Unique_Kmers=Nunique, Length_Contig=Length, Bases, Coverage, Mapped_Reads=Reads, Mapped_Frags=Frags, Total_Mapped_Reads_In_Sample) %>%
  mutate(FUKM=Mapped_Frags/(Unique_Kmers/1e3)/(Total_Mapped_Reads_In_Sample/1e6)) %>%
  group_by(StrainID)%>%
  filter(Unique_Kmers>=quantile(Unique_Kmers,opt$subcontigfilter/100, na.rm=T))%>%
  ungroup()

write_tsv(Norm, paste0(opt$abundances, "/", opt$prefix,".abundances"))

#for plotting with log, add half a read to reads
smallestFUKM <- min(Norm$FUKM[Norm$FUKM > 0], na.rm=TRUE)
plottedNorm <- mutate(Norm, FUKM=(Mapped_Frags/(Unique_Kmers/1e3)/((Total_Mapped_Reads_In_Sample)/1e6))+0.5*smallestFUKM)

#Normalize reads
message(date(), " Plotting normalized data for ", opt$prefix)
pplot<-
  plottedNorm %>%
  ggplot(aes(x=StrainID, y=log10(FUKM), fill=StrainID)) +
  geom_boxplot(outlier.alpha=0) +
  geom_jitter(width=0.2, shape=21) +
  theme_bw() +
  theme(legend.position="none") +
  theme(axis.text.x=element_text(angle=45, hjust=1)) +
  ggtitle(opt$prefix)
ggsave(paste0(opt$abundances, "/", opt$prefix,".pdf"),pplot, device="pdf", height=8, width=11, useDingbats=F )

# create abundance summary file
weighted.percentile <- function(values, weights) {
  weights <- weights[order(values)]
  values <- values[order(values)]
  
  weights <- weights[!is.na(values)]
  values <- values[!is.na(values)]

  weighted_percentiles <- cumsum(weights)/sum(weights)
  percentile_index <- detect_index(weighted_percentiles, function(x) x - opt$weightedpercentile/100 >= 0)
  return(values[percentile_index])
}

abundance_summary <- Norm %>% group_by(StrainID) %>%
  summarise(
    weighted_percentile_FUKM = weighted.percentile(FUKM, Unique_Kmers),
    median_FUKM=median(FUKM, na.rm=T),
    sd_FUKM=sd(FUKM, na.rm=T),
    subcontigs_detected=length(FUKM[FUKM!=0]),
    subcontigs_total=n(),
    percent_detected=length(FUKM[FUKM!=0])/n() * 100
  ) %>%
  mutate(percent_abundance = weighted_percentile_FUKM / sum(weighted_percentile_FUKM) * 100)

write_tsv(abundance_summary, paste0(opt$abundances, "/", opt$prefix,"_abundance_summary.tsv"))
message(date(), " Plotting complete")
